"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var http = require("http");
var fs = require("fs-extra");
var url = require("url");
var http_1 = require("./http");
var NgApimockHandler = (function () {
    function NgApimockHandler() {
        this.MAX_RECORDINGS_PER_MOCK = 2;
    }
    NgApimockHandler.prototype.handleRequest = function (request, response, next, registry, ngApimockId) {
        var _this = this;
        var requestDataChunks = [];
        request.on('data', function (rawData) { return requestDataChunks.push(rawData); });
        request.on('end', function () {
            var payload = Buffer.concat(requestDataChunks).toString();
            var match = _this.getMatchingMock(registry.mocks, request.url, request.method, payload);
            if (match) {
                var selection = _this.getSelection(registry, match.identifier, ngApimockId);
                var variables = _this.getVariables(registry, ngApimockId);
                var mockResponse = match.responses[selection];
                if (mockResponse !== undefined) {
                    if (_this.getEcho(registry, match.identifier, ngApimockId)) {
                        console.log(match.method + ' request made on \'' + match.expression + '\' with payload: ', payload);
                    }
                    var statusCode_1 = mockResponse.status || 200;
                    var jsonCallbackName = _this.getJsonCallbackName(request.url);
                    var headers_1;
                    var chunk_1;
                    if (_this.isBinaryResponse(mockResponse)) {
                        headers_1 = mockResponse.headers || http_1.httpHeaders.CONTENT_TYPE_BINARY;
                        chunk_1 = fs.readFileSync(mockResponse.file);
                    }
                    else {
                        headers_1 = mockResponse.headers || http_1.httpHeaders.CONTENT_TYPE_APPLICATION_JSON;
                        chunk_1 = _this.updateData(mockResponse.data, variables, (match.isArray ? [] : {}));
                    }
                    if (jsonCallbackName !== false) {
                        chunk_1 = jsonCallbackName + '(' + chunk_1 + ')';
                    }
                    var mockDelay = _this.getDelay(registry, match.identifier, ngApimockId);
                    var _delay = mockDelay === null || mockDelay === undefined ? mockResponse.delay : mockDelay;
                    var sendResponse = function () {
                        response.writeHead(statusCode_1, headers_1);
                        response.end(chunk_1);
                        if (registry.record) {
                            _this.storeRecording(payload, chunk_1, request, statusCode_1, registry, match.identifier);
                        }
                    };
                    if (_delay) {
                        setTimeout(sendResponse, _delay);
                    }
                    else {
                        sendResponse();
                    }
                }
                else {
                    if (registry.record && !request.headers.record) {
                        var headers = request.headers;
                        var host = headers.host;
                        var options = {
                            host: host.split(':')[0],
                            port: Number(host.split(':')[1]),
                            path: request.url,
                            method: request.method,
                            headers: headers
                        };
                        headers.record = 'off';
                        var req = http.request(options, function (res) {
                            res.setEncoding('utf8');
                            res.on('data', function (chunk) {
                                _this.storeRecording(payload, chunk.toString('utf8'), request, res.statusCode, registry, match.identifier);
                                response.end(chunk);
                            });
                        });
                        req.on('error', function (e) {
                            response.end(e);
                        });
                        req.end();
                    }
                    else {
                        next();
                    }
                }
            }
            else {
                next();
            }
        });
    };
    NgApimockHandler.prototype.getMatchingMock = function (mocks, requestUrl, method, payload) {
        var allMatches = mocks.filter(function (_mock) {
            var expressionMatches = !!(new RegExp(_mock.expression).exec(decodeURI(requestUrl)));
            var methodMatches = _mock.method === method;
            var bodyMatches = _mock.body ? !!(new RegExp(_mock.body).exec(payload)) : true;
            return expressionMatches && methodMatches && bodyMatches;
        });
        return allMatches.filter(function (_mock) { return _mock.body; })[0] || allMatches[0];
    };
    NgApimockHandler.prototype.isBinaryResponse = function (response) {
        return response.file !== undefined;
    };
    NgApimockHandler.prototype.updateData = function (data, variables, defaults) {
        var _data;
        if (data !== undefined) {
            _data = JSON.stringify(data);
            Object.keys(variables).forEach(function (key) {
                if (variables.hasOwnProperty(key)) {
                    _data = _data.replace(new RegExp('%%' + key + '%%', 'g'), variables[key]);
                }
            });
        }
        else {
            _data = JSON.stringify(defaults);
        }
        return _data;
    };
    NgApimockHandler.prototype.getJsonCallbackName = function (requestUrl) {
        var url_parts = url.parse(requestUrl, true);
        if (!url_parts.query || !url_parts.query.callback) {
            return false;
        }
        return url_parts.query.callback;
    };
    NgApimockHandler.prototype.storeRecording = function (payload, chunk, request, statusCode, registry, identifier) {
        var result = {
            data: typeof chunk === 'string' ? chunk : chunk.toString('utf8'),
            payload: payload,
            datetime: new Date().getTime(),
            method: request.method,
            url: request.url,
            statusCode: statusCode
        };
        if (registry.recordings[identifier] === undefined) {
            registry.recordings[identifier] = [];
        }
        else if (registry.recordings[identifier].length > (this.MAX_RECORDINGS_PER_MOCK - 1)) {
            registry.recordings[identifier].shift();
        }
        registry.recordings[identifier].push(result);
    };
    return NgApimockHandler;
}());
exports.default = NgApimockHandler;
//# sourceMappingURL=ngApimockHandler.js.map